import unified from 'unified';
import markdown from 'remark-parse';
import remark2rehype from 'remark-rehype';
import rehype2react from 'rehype-react';

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var rComponent = /^@\[([-\w]+)\]\(([^()]*)\)/;

function tokenizeComponent(eat, text, silent) {
  var match = rComponent.exec(text);
  if (!match) return null;
  if (silent) return true;

  var _match = _slicedToArray(match, 3),
      all = _match[0],
      component = _match[1],
      value = _match[2];

  return eat(all)({
    type: 'component',
    component: component,
    value: value
  });
}

function parseComponents() {
  var _Parser$prototype = this.Parser.prototype,
      blockTokenizers = _Parser$prototype.blockTokenizers,
      blockMethods = _Parser$prototype.blockMethods;
  blockTokenizers.component = tokenizeComponent;
  blockMethods.unshift('component');
}

var hyphenateRE = /\B([A-Z])/g;

var hyphenate = function hyphenate(str) {
  return str.replace(hyphenateRE, '-$1').toLowerCase();
};

var normalizeKeys = function normalizeKeys(components) {
  var normalized = {};

  for (var key in components) {
    normalized[hyphenate(key)] = components[key];
  }

  return normalized;
}; // convert component nodes to HAST


var handlers = {
  component: function component(toHast, node) {
    return toHast(node, hyphenate(node.component), {
      value: node.value
    });
  }
};
var index = {
  name: 'VueRemark',
  functional: true,
  render: function render(h, context) {
    var _context$slots$defaul = _slicedToArray(context.slots().default, 1),
        slot = _context$slots$defaul[0];

    var components = normalizeKeys(context.props.components || {});

    var createElement = function createElement(tag, attrs, children) {
      var component = components[tag];
      var key = component ? "".concat(tag, "-").concat(attrs.value) : undefined;
      var vnode = h(component || tag, {
        attrs: attrs,
        key: key
      }, children);
      if (!component || vnode.componentOptions) return vnode;
    };

    return unified().use(markdown).use(context.props.plugins).use(parseComponents).use(remark2rehype, {
      handlers: handlers
    }).use(rehype2react, {
      createElement: createElement,
      prefix: false
    }).processSync(slot.text).contents;
  }
};

export default index;
